@@ -99,13 +99,13 @@ public:
 
     explicit FavoritesModel( AbstractDataPluginModelPrivate* d, QObject* parent = 0 );
 
-    virtual int rowCount ( const QModelIndex & parent = QModelIndex() ) const;
+    int rowCount ( const QModelIndex & parent = QModelIndex() ) const override;
 
-    QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
+    QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const override;
 
     void reset();
 
-    QHash<int, QByteArray> roleNames() const;
+    QHash<int, QByteArray> roleNames() const override;
 
 private:
     QHash<int, QByteArray> m_roleNames;
@@ -153,7 +153,7 @@ AbstractDataPluginModelPrivate::~AbstractDataPluginModelPrivate() {
 void AbstractDataPluginModelPrivate::updateFavoriteItems()
 {
     if ( m_favoriteItemsOnly ) {
-        foreach( const QString &id, m_favoriteItems ) {
+        for( const QString &id: m_favoriteItems ) {
             if ( !m_parent->findItem( id ) ) {
                 m_parent->getItem( id );
             }
@@ -210,7 +210,7 @@ int FavoritesModel::rowCount ( const QModelIndex &parent ) const
     }
 
     int count = 0;
-    foreach( AbstractDataPluginItem* item, d->m_itemSet ) {
+    for( AbstractDataPluginItem* item: d->m_itemSet ) {
         if ( item->initialized() && item->isFavorite() ) {
             ++count;
         }
@@ -224,7 +224,7 @@ QVariant FavoritesModel::data( const QModelIndex &index, int role ) const
     int const row = index.row();
     if ( row >= 0 && row < rowCount() ) {
         int count = 0;
-        foreach( AbstractDataPluginItem* item, d->m_itemSet ) {
+        for( AbstractDataPluginItem* item: d->m_itemSet ) {
             if ( item->initialized() && item->isFavorite() ) {
                 if ( count == row ) {
                     QString const roleName = roleNames().value( role );
@@ -292,8 +292,8 @@ QList<AbstractDataPluginItem*> AbstractDataPluginModel::items( const ViewportPar
 
     if ( d->m_needsSorting ) {
         // Both the candidates list and the list of all items need to be sorted
-        qSort( candidates.begin(), candidates.end(), lessThanByPointer );
-        qSort( d->m_itemSet.begin(), d->m_itemSet.end(), lessThanByPointer );
+        std::sort( candidates.begin(), candidates.end(), lessThanByPointer );
+        std::sort( d->m_itemSet.begin(), d->m_itemSet.end(), lessThanByPointer );
         d->m_needsSorting =  false;
     }
 
@@ -328,8 +328,8 @@ QList<AbstractDataPluginItem*> AbstractDataPluginModel::items( const ViewportPar
             bool collides = false;
             int const length = list.length();
             for ( int j=0; !collides && j<length; ++j ) {
-                foreach( const QRectF &rect, list[j]->boundingRects() ) {
-                    foreach( const QRectF &itemRect, (*i)->boundingRects() ) {
+                for( const QRectF &rect: list[j]->boundingRects() ) {
+                    for( const QRectF &itemRect: (*i)->boundingRects() ) {
                         if ( rect.intersects( itemRect ) )
                             collides = true;
                     }
@@ -360,7 +360,7 @@ QList<AbstractDataPluginItem *> AbstractDataPluginModel::whichItemAt( const QPoi
     QList<AbstractDataPluginItem *> itemsAt;
 
     const QPointF curposF(curpos);
-    foreach( AbstractDataPluginItem* item, d->m_displayedItems ) {
+    for( AbstractDataPluginItem* item: d->m_displayedItems ) {
         if (item && item->contains(curposF)) {
             itemsAt.append( item );
         }
@@ -408,7 +408,7 @@ void AbstractDataPluginModel::addItemsToList( const QList<AbstractDataPluginItem
 {
     bool needsUpdate = false;
     bool favoriteChanged = false;
-    foreach( AbstractDataPluginItem *item, items ) {
+    for( AbstractDataPluginItem *item: items ) {
         if( !item ) {
             continue;
         }
@@ -426,7 +426,7 @@ void AbstractDataPluginModel::addItemsToList( const QList<AbstractDataPluginItem
         mDebug() << "New item " << item->id();
 
         // This find the right position in the sorted to insert the new item
-        QList<AbstractDataPluginItem*>::iterator i = qLowerBound( d->m_itemSet.begin(),
+        QList<AbstractDataPluginItem*>::iterator i = std::lower_bound( d->m_itemSet.begin(),
                                                                   d->m_itemSet.end(),
                                                                   item,
                                                                   lessThanByPointer );
@@ -540,7 +540,7 @@ QString AbstractDataPluginModelPrivate::generateFilepath( const QString& id, con
 
 AbstractDataPluginItem *AbstractDataPluginModel::findItem( const QString& id ) const
 {
-    foreach ( AbstractDataPluginItem *item, d->m_itemSet ) {
+    for ( AbstractDataPluginItem *item: d->m_itemSet ) {
         if( item->id() == id ) {
             return item;
         }
