@@ -18,11 +18,14 @@
 #include <QPointer>
 
 // Marble
+#include "GeoDataLatLonAltBox.h"
 #include "GeoDataContainer.h"
 #include "GeoDataDocument.h"
+#include "GeoDataDocumentWriter.h"
 #include "GeoDataPlacemark.h"
 #include "GeoDataTreeModel.h"
 #include "FileManager.h"
+#include "KmlElementDictionary.h"
 #include "MarblePlacemarkModel.h"
 #include "MarbleModel.h"
 #include "MarbleWidget.h"
@@ -143,7 +146,8 @@ void FileViewWidgetPrivate::saveFile()
         = index.model()->data( index, MarblePlacemarkModel::ObjectPointerRole ).value<GeoDataObject*>();
     GeoDataDocument *document = dynamic_cast<GeoDataDocument*>(object);
     if ( document && !document->fileName().isEmpty() ) {
-        m_fileManager->saveFile( QFileDialog::getSaveFileName( q, "Select filename for KML document" ), document );
+        const QString saveFileName = QFileDialog::getSaveFileName(q, QObject::tr("Select filename for KML document"));
+        GeoDataDocumentWriter::write(saveFileName, *document, kml::kmlTag_nameSpaceOgc22);
     }
 }
 
