@@ -26,6 +26,9 @@
 #include "GeoDataExtendedData.h"
 #include "GeoDataFolder.h"
 #include "GeoDataPlacemark.h"
+#include "GeoDataPoint.h"
+#include "GeoDataPolygon.h"
+#include "GeoDataLinearRing.h"
 #include "GeoDataLookAt.h"
 #include "GeoDataMultiGeometry.h"
 #include "GeoDataPlaylist.h"
@@ -36,7 +39,6 @@
 #include "GeoDataStyle.h"
 #include "GeoDataIconStyle.h"
 #include "GeoDataListStyle.h"
-#include "GeoDataTypes.h"
 #include "FileManager.h"
 #include "MarbleDebug.h"
 #include "MarblePlacemarkModel.h"
@@ -90,10 +92,8 @@ GeoDataTreeModel::Private::~Private()
 
 void GeoDataTreeModel::Private::checkParenting( GeoDataObject *object )
 {
-    if(    object->nodeType() == GeoDataTypes::GeoDataDocumentType
-        || object->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-        GeoDataContainer *container = static_cast<GeoDataContainer*>( object );
-        foreach( GeoDataFeature *child, container->featureList() ) {
+    if (const auto container = dynamic_cast<const GeoDataContainer *>(object)) {
+        for( GeoDataFeature *child: container->featureList() ) {
             if ( child->parent() != container ) {
                 qWarning() << "Parenting mismatch for " << child->name();
                 Q_ASSERT( 0 );
@@ -117,56 +117,10 @@ GeoDataTreeModel::~GeoDataTreeModel()
     delete d;
 }
 
-bool GeoDataTreeModel::hasChildren( const QModelIndex &parent ) const
-{
-    GeoDataObject *parentItem;
-    if ( parent.column() > 0 ) {
-        return false;
-    }
-
-    if ( !parent.isValid() ) {
-        parentItem = d->m_rootDocument;
-    } else {
-        parentItem = static_cast<GeoDataObject*>( parent.internalPointer() );
-    }
-
-    if ( !parentItem ) {
-        return false;
-    }
-
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-        GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( parentItem );
-        return dynamic_cast<const GeoDataMultiGeometry*>( placemark->geometry() );
-    }
-
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataFolderType
-         || parentItem->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-        GeoDataContainer *container = static_cast<GeoDataContainer*>( parentItem );
-        return container->size();
-    }
-
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) {
-        GeoDataMultiGeometry *geometry = static_cast<GeoDataMultiGeometry*>( parentItem );
-        return geometry->size();
-    }
-
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataTourType ) {
-        GeoDataTour *tour = static_cast<GeoDataTour*>( parentItem );
-        return tour->playlist();
-    }
-
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentItem );
-        return playlist->size();
-    }
-
-    return false;
-}
-
 int GeoDataTreeModel::rowCount( const QModelIndex &parent ) const
 {
 //    mDebug() << "rowCount";
-    GeoDataObject *parentItem;
+    const GeoDataObject *parentItem;
     if ( parent.column() > 0 ) {
 //        mDebug() << "rowCount bad column";
         return 0;
@@ -176,7 +130,7 @@ int GeoDataTreeModel::rowCount( const QModelIndex &parent ) const
 //        mDebug() << "rowCount root parent";
         parentItem = d->m_rootDocument;
     } else {
-        parentItem = static_cast<GeoDataObject*>( parent.internalPointer() );
+        parentItem = static_cast<const GeoDataObject *>(parent.internalPointer());
     }
 
     if ( !parentItem ) {
@@ -184,42 +138,36 @@ int GeoDataTreeModel::rowCount( const QModelIndex &parent ) const
         return 0;
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataFolderType
-         || parentItem->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-        GeoDataContainer *container = static_cast<GeoDataContainer*>( parentItem );
+    if (const GeoDataContainer *container = dynamic_cast<const GeoDataContainer *>(parentItem)) {
 //        mDebug() << "rowCount " << type << "(" << parentItem << ") =" << container->size();
         return container->size();
 //    } else {
 //        mDebug() << "rowCount bad container " << container;
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-        GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( parentItem );
+    if (const auto placemark = geodata_cast<GeoDataPlacemark>(parentItem)) {
         if ( dynamic_cast<const GeoDataMultiGeometry*>( placemark->geometry() ) ) {
 //            mDebug() << "rowCount " << type << "(" << parentItem << ") = 1";
             return 1;
         }
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) {
-        GeoDataMultiGeometry *geometry = static_cast<GeoDataMultiGeometry*>( parentItem );
+    if (const auto geometry = geodata_cast<GeoDataMultiGeometry>(parentItem)) {
 //        mDebug() << "rowCount " << parent << " " << type << " " << geometry->size();
         return geometry->size();
 //    } else {
 //        mDebug() << "rowCount bad geometry " << geometry;
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataTourType ) {
-        GeoDataTour *tour = static_cast<GeoDataTour*>( parentItem );
-        GeoDataPlaylist *playlist = tour->playlist();
+    if (const auto tour = geodata_cast<GeoDataTour>(parentItem)) {
+        const GeoDataPlaylist *playlist = tour->playlist();
         if ( playlist ) {
 //            mDebug() << "rowCount " << parent << " Playlist " << 1;
             return 1;
         }
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentItem );
+    if (const auto playlist = geodata_cast<GeoDataPlaylist>(parentItem)) {
 //         mDebug() << "rowCount " << parent << " Playlist " << playlist->size();
         return playlist->size();
     }
@@ -265,8 +213,7 @@ QVariant GeoDataTreeModel::data( const QModelIndex &index, int role ) const
     GeoDataObject *object = static_cast<GeoDataObject*>( index.internalPointer() );
     if ( role == Qt::DisplayRole ) {
 
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( object );
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
                 if ( index.column() == 0 ){
                     if ( placemark->countryCode().isEmpty() ) {
                         return QVariant( placemark->name() );
@@ -285,10 +232,8 @@ QVariant GeoDataTreeModel::data( const QModelIndex &index, int role ) const
                     return QVariant( placemark->zoomLevel() );
                 }
         }
-        if ( object->nodeType() == GeoDataTypes::GeoDataFolderType
-             || object->nodeType() == GeoDataTypes::GeoDataDocumentType
-             || object->nodeType() == GeoDataTypes::GeoDataTourType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
+
+        if (const auto feature = dynamic_cast<const GeoDataFeature *>(object)) {
             if ( index.column() == 0 ){
                 return QVariant( feature->name() );
             }
@@ -315,11 +260,8 @@ QVariant GeoDataTreeModel::data( const QModelIndex &index, int role ) const
     }
     else if ( role == Qt::CheckStateRole
               && index.column() == 0 ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *feature = static_cast<GeoDataPlacemark*>( object );
-            const char* type = feature->geometry()->nodeType();
-            if ( feature->parent()->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-                GeoDataFolder *folder = static_cast<GeoDataFolder *>( feature->parent() );
+        if (const auto feature = geodata_cast<GeoDataPlacemark>(object)) {
+            if (const auto folder = geodata_cast<GeoDataFolder>(feature->parent())) {
                 if ( folder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder 
                   || folder->style()->listStyle().listItemType() == GeoDataListStyle::CheckOffOnly) {
                     if ( feature->isVisible() ) {
@@ -329,25 +271,18 @@ QVariant GeoDataTreeModel::data( const QModelIndex &index, int role ) const
                     }
                 }
             }
-            if ( type == GeoDataTypes::GeoDataLineStringType
-                 || type == GeoDataTypes::GeoDataPolygonType
-                 || type == GeoDataTypes::GeoDataLinearRingType
-                 || type == GeoDataTypes::GeoDataMultiGeometryType
-                 || type == GeoDataTypes::GeoDataTrackType
-                 ) {
-                if ( feature->isGloballyVisible() ) {
-                    return QVariant( Qt::Checked );
-                } else if ( feature->isVisible() ) {
-                    return QVariant( Qt::PartiallyChecked );
-                } else {
-                    return QVariant( Qt::Unchecked );
-                }
+
+            if (feature->isGloballyVisible()) {
+                return QVariant(Qt::Checked);
             }
-        } else if ( object->nodeType() == GeoDataTypes::GeoDataFolderType
-                    || object->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
-            if ( object->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-                GeoDataFolder *folder = static_cast<GeoDataFolder *>( object );
+
+            if (feature->isVisible()) {
+                return QVariant(Qt::PartiallyChecked);
+            }
+
+            return QVariant(Qt::Unchecked);
+        } else if (auto feature = dynamic_cast<GeoDataContainer *>(object)) {
+            if (auto folder = geodata_cast<GeoDataFolder>(object)) {
                 if ( folder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder) {
                     bool anyVisible = false;
                     QVector<GeoDataFeature *>::Iterator i = folder->begin();
@@ -393,62 +328,46 @@ QVariant GeoDataTreeModel::data( const QModelIndex &index, int role ) const
     }
     else if ( role == Qt::DecorationRole
               && index.column() == 0 ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-             || object->nodeType() == GeoDataTypes::GeoDataFolderType
-             || object->nodeType() == GeoDataTypes::GeoDataDocumentType
-             || object->nodeType() == GeoDataTypes::GeoDataTourType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
-            if (feature->style()->iconStyle().icon().isNull()) return QImage();
+        if (const auto feature = dynamic_cast<const GeoDataFeature *>(object)) {
+            if (feature->style()->iconStyle().icon().isNull()) {
+                return QImage();
+            }
+
             return QVariant(feature->style()->iconStyle().icon().scaled( QSize(16,16), Qt::KeepAspectRatio, Qt::SmoothTransformation ));
 	    }
     } else if ( role == Qt::ToolTipRole
               && index.column() == 0 ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-             || object->nodeType() == GeoDataTypes::GeoDataFolderType
-             || object->nodeType() == GeoDataTypes::GeoDataDocumentType
-             || object->nodeType() == GeoDataTypes::GeoDataTourType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
+        if (const auto feature = dynamic_cast<const GeoDataFeature *>(object)) {
             return QVariant( feature->description() );
         }
     } else if ( role == MarblePlacemarkModel::ObjectPointerRole ) {
         return qVariantFromValue( object );
     } else if ( role == MarblePlacemarkModel::PopularityIndexRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( object );
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
             return QVariant( placemark->zoomLevel() );
         }
     } else if ( role == MarblePlacemarkModel::PopularityRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( object );
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
             return QVariant( placemark->popularity() );
         }
     } else if ( role == MarblePlacemarkModel::CoordinateRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( object );
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
             return qVariantFromValue( placemark->coordinate() );
-        } else if ( object->nodeType() == GeoDataTypes::GeoDataFlyToType ) {
-            GeoDataFlyTo *flyTo = static_cast<GeoDataFlyTo*>( object );
-            if ( flyTo->view() && flyTo->view()->nodeType() == GeoDataTypes::GeoDataCameraType )  {
-                GeoDataCamera *camera = static_cast<GeoDataCamera*>( flyTo->view() );
+        } else if (const auto flyTo = geodata_cast<GeoDataFlyTo>(object)) {
+            if (const auto camera = geodata_cast<GeoDataCamera>(flyTo->view())) {
                 return QVariant::fromValue<GeoDataCoordinates>( camera->coordinates() );
-            } else if ( flyTo->view() && flyTo->view()->nodeType() == GeoDataTypes::GeoDataLookAtType )  {
-                GeoDataLookAt *lookAt = static_cast<GeoDataLookAt*>( flyTo->view() );
+            } else if (const auto lookAt = (flyTo->view() ? geodata_cast<GeoDataLookAt>(flyTo->view()) : 0)) {
                 return QVariant::fromValue<GeoDataCoordinates>( lookAt->coordinates() );
             }
         }
     } else if ( role == Qt::BackgroundRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( object );
-            if ( placemark->parent() &&
-                 ( placemark->parent()->nodeType() == GeoDataTypes::GeoDataFolderType ||
-                   placemark->parent()->nodeType() == GeoDataTypes::GeoDataDocumentType ) ) {
-                GeoDataContainer *container = static_cast<GeoDataContainer *>( placemark->parent() );
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
+            if (const GeoDataContainer *container = dynamic_cast<const GeoDataContainer *>(placemark->parent())) {
                 return container->customStyle() ? QVariant( QBrush( container->customStyle()->listStyle().backgroundColor() )) : QVariant();
             }
         }
     } else if (role == MarblePlacemarkModel::IconPathRole) {
-        if (object->nodeType() == GeoDataTypes::GeoDataPlacemarkType) {
-            GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>(object);
+        if (const auto placemark = geodata_cast<GeoDataPlacemark>(object)) {
             return placemark->style()->iconStyle().iconPath();
         }
     }
@@ -479,35 +398,29 @@ QModelIndex GeoDataTreeModel::index( int row, int column, const QModelIndex &par
     GeoDataObject *childItem = 0;
 
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataFolderType
-         || parentItem->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-        GeoDataContainer *container = static_cast<GeoDataContainer*>( parentItem );
+    if (auto container = dynamic_cast<GeoDataContainer *>(parentItem)) {
         childItem = container->child( row );
         return createIndex( row, column, childItem );
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-        GeoDataPlacemark *placemark = static_cast<GeoDataPlacemark*>( parentItem );
+    if (const auto placemark = geodata_cast<GeoDataPlacemark>(parentItem)) {
         childItem = placemark->geometry();
         if ( dynamic_cast<GeoDataMultiGeometry*>( childItem ) ) {
             return createIndex( row, column, childItem );
         }
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) {
-        GeoDataMultiGeometry *geometry = static_cast<GeoDataMultiGeometry*>( parentItem );
+    if (const auto geometry = geodata_cast<GeoDataMultiGeometry>(parentItem)) {
         childItem = geometry->child( row );
         return createIndex( row, column, childItem );
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataTourType ) {
-        GeoDataTour *tour = static_cast<GeoDataTour*>( parentItem );
+    if (const auto tour = geodata_cast<GeoDataTour>(parentItem)) {
         childItem = tour->playlist();
         return createIndex( row, column, childItem );
     }
 
-    if ( parentItem->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentItem );
+    if (const auto playlist = geodata_cast<GeoDataPlaylist>(parentItem)) {
         childItem = playlist->primitive( row );
         return createIndex(row, column, childItem);
     }
@@ -543,9 +456,7 @@ QModelIndex GeoDataTreeModel::parent( const QModelIndex &index ) const
         }
 
         // greatParent can be a container
-        if ( greatParentObject->nodeType() == GeoDataTypes::GeoDataFolderType
-             || greatParentObject->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-            GeoDataContainer *greatparentContainer = static_cast<GeoDataContainer*>( greatParentObject );
+        if (auto greatparentContainer = dynamic_cast<GeoDataContainer *>(greatParentObject)) {
             GeoDataFeature *parentFeature = static_cast<GeoDataFeature*>( parentObject );
 //            mDebug() << "parent " << childObject->nodeType() << "(" << childObject << ") = "
 //                    << parentObject->nodeType() << "[" << greatparentContainer->childPosition( parentFeature ) << "](" << parentObject << ")";
@@ -553,7 +464,7 @@ QModelIndex GeoDataTreeModel::parent( const QModelIndex &index ) const
         }
 
         // greatParent can be a placemark
-        if ( greatParentObject->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
+        if (geodata_cast<GeoDataPlacemark>(greatParentObject)) {
 //            GeoDataPlacemark *greatparentPlacemark = static_cast<GeoDataPlacemark*>( greatParentObject );
 //                mDebug() << "parent " << childObject->nodeType() << "(" << childObject << ") = "
 //                        << parentObject->nodeType() << "[0](" << parentObject << ")";
@@ -561,16 +472,14 @@ QModelIndex GeoDataTreeModel::parent( const QModelIndex &index ) const
         }
 
         // greatParent can be a multigeometry
-        if ( greatParentObject->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) {
-            GeoDataMultiGeometry *greatparentMultiGeo = static_cast<GeoDataMultiGeometry*>( greatParentObject );
+        if (GeoDataMultiGeometry *greatparentMultiGeo = geodata_cast<GeoDataMultiGeometry>(greatParentObject)) {
             GeoDataGeometry *parentGeometry = static_cast<GeoDataGeometry*>( parentObject );
 //                mDebug() << "parent " << childObject->nodeType() << "(" << childObject << ") = "
 //                        << parentObject->nodeType() << "[" << greatParentItem->childPosition( parentGeometry ) << "](" << parentObject << ")";
             return createIndex( greatparentMultiGeo->childPosition( parentGeometry ), 0, parentObject );
         }
 
-        if ( greatParentObject->nodeType() == GeoDataTypes::GeoDataTourType ) {
-            GeoDataTour *tour = static_cast<GeoDataTour*>( greatParentObject );
+        if (GeoDataTour *tour = geodata_cast<GeoDataTour>(greatParentObject)) {
             return createIndex( 0, 0, tour->playlist() );
         }
 
@@ -592,13 +501,9 @@ bool GeoDataTreeModel::setData ( const QModelIndex & index, const QVariant & val
 
     GeoDataObject *object = static_cast<GeoDataObject*>( index.internalPointer() );
     if ( role == Qt::CheckStateRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-             || object->nodeType() == GeoDataTypes::GeoDataFolderType
-             || object->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
+        if (auto feature = dynamic_cast<GeoDataFeature *>(object)) {
             bool bValue = value.toBool();
-            if (feature->parent()->nodeType() == GeoDataTypes::GeoDataFolderType) {
-                GeoDataFolder *pfolder = static_cast<GeoDataFolder *>(feature->parent());
+            if (auto pfolder = geodata_cast<GeoDataFolder>(feature->parent())) {
                 if ( pfolder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder) {
                     if ( bValue ) {
                         QVector< GeoDataFeature * >::Iterator i = pfolder->begin();
@@ -608,8 +513,7 @@ bool GeoDataTreeModel::setData ( const QModelIndex & index, const QVariant & val
                     }
                 }
             }
-            if ( feature->nodeType() == GeoDataTypes::GeoDataFolderType) {
-                GeoDataFolder *folder = static_cast<GeoDataFolder *>( object );
+            if (auto folder = geodata_cast<GeoDataFolder>(object)) {
                 if ( bValue ) {
                 } else {
                     if ( folder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder 
@@ -628,11 +532,7 @@ bool GeoDataTreeModel::setData ( const QModelIndex & index, const QVariant & val
             return true;
         }
     } else if ( role == Qt::EditRole ) {
-        if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-             || object->nodeType() == GeoDataTypes::GeoDataFolderType
-             || object->nodeType() == GeoDataTypes::GeoDataDocumentType
-             || object->nodeType() == GeoDataTypes::GeoDataTourType ) {
-            GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
+        if (auto feature = dynamic_cast<GeoDataFeature *>(object)) {
             feature->setName( value.toString() );
             mDebug() << "setData " << feature->name() << " " << value.toString();
             updateFeature( feature );
@@ -648,19 +548,12 @@ Qt::ItemFlags GeoDataTreeModel::flags ( const QModelIndex & index ) const
     if ( !index.isValid() )
         return Qt::NoItemFlags;
 
-    GeoDataObject *object = static_cast<GeoDataObject*>( index.internalPointer() );
-    if ( object->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-        GeoDataDocument *document = static_cast<GeoDataDocument*>( object );
-        if( document->documentRole() == UserDocument ) {
-            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
-        }
-    }
-    if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) {
-        GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
-        GeoDataObject *parent = feature->parent();
+    const GeoDataObject *object = static_cast<const GeoDataObject *>(index.internalPointer());
 
-        if ( parent->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-            GeoDataFolder *parentfolder = static_cast<GeoDataFolder *>( parent );
+    if (const auto feature = geodata_cast<GeoDataPlacemark>(object)) {
+        const GeoDataObject *parent = feature->parent();
+
+        if (const auto parentfolder = geodata_cast<GeoDataFolder>(parent)) {
             if ( parentfolder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder ) {
                 return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
             } else if ( parentfolder->style()->listStyle().listItemType() == GeoDataListStyle::CheckHideChildren ) {
@@ -668,14 +561,14 @@ Qt::ItemFlags GeoDataTreeModel::flags ( const QModelIndex & index ) const
             }
         }
     }
-    if ( object->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-        GeoDataFolder *folder = static_cast<GeoDataFolder *>( object );
+
+    if (const auto folder = geodata_cast<GeoDataFolder>(object)) {
         if ( folder->style()->listStyle().listItemType() == GeoDataListStyle::RadioFolder) {
             return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable; 
         } else if ( folder->style()->listStyle().listItemType() == GeoDataListStyle::CheckOffOnly ) {
-            QVector<GeoDataFeature *>::Iterator i = folder->begin();
+            QVector<GeoDataFeature *>::ConstIterator i = folder->constBegin();
             bool allVisible = true;
-            for(; i < folder->end(); ++i) {
+            for (; i < folder->constEnd(); ++i) {
                 if( ! (*i)->isVisible() ) {
                     allVisible = false;
                     break;
@@ -690,39 +583,35 @@ Qt::ItemFlags GeoDataTreeModel::flags ( const QModelIndex & index ) const
             return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable; 
         }
     }
-    if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-     || object->nodeType() == GeoDataTypes::GeoDataFolderType ) {
-        GeoDataFeature *feature = static_cast<GeoDataFeature*>( object );
-        GeoDataObject *parent = feature->parent();
-        while( parent->nodeType() != GeoDataTypes::GeoDataDocumentType ) {
-            parent = parent->parent();
-        }
-        GeoDataDocument *document = static_cast<GeoDataDocument*>( parent );
-        if( document->documentRole() == UserDocument ) {
-            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;;
-        }
-    }
 
-    if ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType
-         || object->nodeType() == GeoDataTypes::GeoDataFolderType
-         || object->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-        return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable;
-    }
-
-    if ( object->nodeType() == GeoDataTypes::GeoDataTourType ) {
+    if (geodata_cast<GeoDataTour>(object)) {
         return Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsEnabled;
     }
 
-    if ( object->nodeType() == GeoDataTypes::GeoDataWaitType
-         || object->nodeType() == GeoDataTypes::GeoDataFlyToType
-         || object->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
+    if (dynamic_cast<const GeoDataFeature *>(object)) {
+        const GeoDataObject *parent = object;
+        while (!geodata_cast<GeoDataDocument>(parent)) {
+            parent = parent->parent();
+        }
+        const GeoDataDocument *document = static_cast<const GeoDataDocument *>(parent);
+        if( document->documentRole() == UserDocument ) {
+            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;;
+        }
+
+        return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable;
+    }
+
+    if (geodata_cast<GeoDataWait>(object)
+         || geodata_cast<GeoDataFlyTo>(object)
+         || geodata_cast<GeoDataPlaylist>(object)) {
         return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
     }
+
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
 
 
-QModelIndex GeoDataTreeModel::index( GeoDataObject *object ) const
+QModelIndex GeoDataTreeModel::index(const GeoDataObject *object) const
 {
     if ( object == 0 )
         return QModelIndex();
@@ -734,30 +623,30 @@ QModelIndex GeoDataTreeModel::index( GeoDataObject *object ) const
     //and Geometries that are children of MultiGeometries
     //You can not call this function with an element that does not belong to the tree
 
-    Q_ASSERT( ( object->nodeType() == GeoDataTypes::GeoDataFolderType )
-              || ( object->nodeType() == GeoDataTypes::GeoDataDocumentType )
-              || ( object->nodeType() == GeoDataTypes::GeoDataPlacemarkType )
-              || ( object->nodeType() == GeoDataTypes::GeoDataTourType )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataPlaylistType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataTourType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataWaitType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataPlaylistType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataFlyToType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataPlaylistType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataLineStringType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataLinearRingType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataPointType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) )
-              || ( ( object->nodeType() == GeoDataTypes::GeoDataPolygonType )
-                   && ( object->parent()->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) )
-              || ( object->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) );
+    Q_ASSERT(geodata_cast<GeoDataFolder>(object)
+              || geodata_cast<GeoDataDocument>(object)
+              || geodata_cast<GeoDataPlacemark>(object)
+              || geodata_cast<GeoDataTour>(object)
+              || ( geodata_cast<GeoDataPlaylist>(object)
+                   && geodata_cast<GeoDataTour>(object->parent()))
+              || (geodata_cast<GeoDataWait>(object)
+                   && geodata_cast<GeoDataPlaylist>(object->parent()))
+              || ( geodata_cast<GeoDataFlyTo>(object)
+                   && geodata_cast<GeoDataPlaylist>(object->parent()))
+              || (geodata_cast<GeoDataLineString>(object)
+                   && geodata_cast<GeoDataMultiGeometry>(object->parent()))
+              || (geodata_cast<GeoDataLinearRing>(object)
+                   && geodata_cast<GeoDataMultiGeometry>(object->parent()))
+              || (geodata_cast<GeoDataPoint>(object)
+                   && geodata_cast<GeoDataMultiGeometry>(object->parent()))
+              || (geodata_cast<GeoDataPolygon>(object)
+                   && geodata_cast<GeoDataMultiGeometry>(object->parent()))
+              || geodata_cast<GeoDataMultiGeometry>(object));
 
 
-    QList< GeoDataObject* > ancestors;
+    QList<const GeoDataObject *> ancestors;
 
-    GeoDataObject *itup = object; //Iterator to reach the top of the GeoDataDocument (bottom-up)
+    const GeoDataObject *itup = object; //Iterator to reach the top of the GeoDataDocument (bottom-up)
 
     while ( itup && ( itup != d->m_rootDocument ) ) {//We reach up to the rootDocument
 
@@ -768,35 +657,33 @@ QModelIndex GeoDataTreeModel::index( GeoDataObject *object ) const
     QModelIndex itdown;
     if ( !ancestors.isEmpty() ) {
 
-        itdown = index( d->m_rootDocument->childPosition( static_cast<GeoDataFeature*>( ancestors.last() ) ),0,QModelIndex());//Iterator to go top down
+        itdown = index(d->m_rootDocument->childPosition(static_cast<const GeoDataFeature *>(ancestors.last())), 0, QModelIndex());//Iterator to go top down
 
         while ( ( ancestors.size() > 1 ) ) {
 
-            GeoDataObject *parent = static_cast<GeoDataObject*>( ancestors.last() );
+            const GeoDataObject *parent = static_cast<const GeoDataObject*>(ancestors.last());
 
-            if ( ( parent->nodeType() == GeoDataTypes::GeoDataFolderType )
-                || ( parent->nodeType() == GeoDataTypes::GeoDataDocumentType ) ) {
+            if (const auto container = dynamic_cast<const GeoDataContainer *>(parent)) {
 
                 ancestors.removeLast();
-                itdown = index( static_cast<GeoDataContainer*>(parent)->childPosition( static_cast<GeoDataFeature*>( ancestors.last() ) ) , 0, itdown );
-            } else if ( ( parent->nodeType() == GeoDataTypes::GeoDataPlacemarkType ) ) {
+                itdown = index(container->childPosition(static_cast<const GeoDataFeature *>(ancestors.last())), 0, itdown);
+            } else if (geodata_cast<GeoDataPlacemark>(parent)) {
                 //The only child of the model is a Geometry or MultiGeometry object
                 //If it is a geometry object, we should be on the bottom of the list
                 ancestors.removeLast();
-                if( ancestors.last()->nodeType() == GeoDataTypes::GeoDataMultiGeometryType )
+                if (geodata_cast<GeoDataMultiGeometry>(ancestors.last()))
                     itdown = index( 0 , 0, itdown );
                 else
                     itdown = QModelIndex();
 
-            }  else if ( ( parent->nodeType() == GeoDataTypes::GeoDataMultiGeometryType ) ) {
+            }  else if (auto multiGeometry = geodata_cast<GeoDataMultiGeometry>(parent)) {
                 //The child is one of the geometry children of MultiGeometry
                 ancestors.removeLast();
-                itdown = index( static_cast<GeoDataMultiGeometry*>(parent)->childPosition( static_cast<GeoDataGeometry*>(ancestors.last()) ) , 0, itdown );
-            } else if ( ( parent->nodeType() == GeoDataTypes::GeoDataTourType ) ) {
+                itdown = index(multiGeometry->childPosition(static_cast<const GeoDataGeometry *>(ancestors.last())), 0, itdown);
+            } else if (geodata_cast<GeoDataTour>(parent)) {
                 ancestors.removeLast();
                 itdown = index( 0, 0, itdown );
-            } else if ( ( parent->nodeType() == GeoDataTypes::GeoDataPlaylistType ) ) {
-                GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parent );
+            } else if (auto playlist = geodata_cast<GeoDataPlaylist>(parent)) {
                 for ( int i=0; i< playlist->size(); i++ )
                 {
                     if ( playlist->primitive(i) == ancestors.last() )
@@ -868,7 +755,7 @@ bool GeoDataTreeModel::removeFeature( GeoDataContainer *parent, int row )
     return false; //Tried to remove a row that is not contained in the parent.
 }
 
-int GeoDataTreeModel::removeFeature( const GeoDataFeature *feature )
+int GeoDataTreeModel::removeFeature(GeoDataFeature *feature)
 {
     if ( feature && ( feature!=d->m_rootDocument ) )  {
 
@@ -881,8 +768,7 @@ int GeoDataTreeModel::removeFeature( const GeoDataFeature *feature )
 
         GeoDataObject *parent = static_cast< GeoDataObject* >( feature->parent() );
 
-        if ( ( parent->nodeType() == GeoDataTypes::GeoDataFolderType )
-            || ( parent->nodeType() == GeoDataTypes::GeoDataDocumentType ) ) {
+        if (dynamic_cast<const GeoDataContainer *>(parent)) {
 
             int row = static_cast< GeoDataContainer* >( feature->parent() )->childPosition( feature );
             if ( row != -1 ) {
@@ -935,8 +821,7 @@ GeoDataDocument * GeoDataTreeModel::rootDocument()
 int GeoDataTreeModel::addTourPrimitive( const QModelIndex &parent, GeoDataTourPrimitive *primitive, int row )
 {
     GeoDataObject *parentObject = static_cast<GeoDataObject*>( parent.internalPointer() );
-    if( parent.isValid() && parentObject->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentObject );
+    if (auto playlist = geodata_cast<GeoDataPlaylist>(parentObject)) {
         if( row == -1 ) {
             row = playlist->size();
         }
@@ -951,8 +836,7 @@ int GeoDataTreeModel::addTourPrimitive( const QModelIndex &parent, GeoDataTourPr
 bool GeoDataTreeModel::removeTourPrimitive( const QModelIndex &parent , int index)
 {
     GeoDataObject *parentObject = static_cast<GeoDataObject*>( parent.internalPointer() );
-    if( parent.isValid() && parentObject->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentObject );
+    if (auto playlist = (parent.isValid() ? geodata_cast<GeoDataPlaylist>(parentObject) : 0)) {
         if( playlist->size() > index ) {
             beginRemoveRows( parent, index, index );
             playlist->removePrimitiveAt( index );
@@ -966,8 +850,7 @@ bool GeoDataTreeModel::removeTourPrimitive( const QModelIndex &parent , int inde
 bool GeoDataTreeModel::swapTourPrimitives( const QModelIndex &parent, int indexA, int indexB )
 {
     GeoDataObject *parentObject = static_cast<GeoDataObject*>( parent.internalPointer() );
-    if( parent.isValid() && parentObject->nodeType() == GeoDataTypes::GeoDataPlaylistType ) {
-        GeoDataPlaylist *playlist = static_cast<GeoDataPlaylist*>( parentObject );
+    if (auto playlist = (parent.isValid() ? geodata_cast<GeoDataPlaylist>(parentObject) : 0)) {
         if( indexA > indexB ) {
             qSwap(indexA, indexB);
         }
