@@ -31,9 +31,10 @@ GeoPhotoGraphicsItem::GeoPhotoGraphicsItem( const GeoDataFeature *feature )
     }
 }
 
-void GeoPhotoGraphicsItem::paint(GeoPainter* painter, const ViewportParams* viewport , const QString &layer)
+void GeoPhotoGraphicsItem::paint(GeoPainter* painter, const ViewportParams* viewport , const QString &layer, int tileZoomLevel)
 {
     Q_UNUSED(layer);
+    Q_UNUSED(tileZoomLevel);
     /* The code below loads the image lazily (only
     * when it will actually be displayed). Once it was
     * loaded but moves out of the viewport, it is unloaded
@@ -68,6 +69,28 @@ const GeoDataLatLonAltBox& GeoPhotoGraphicsItem::latLonAltBox() const
     return m_point.latLonAltBox();
 }
 
+bool GeoPhotoGraphicsItem::contains(const QPoint &curpos, const ViewportParams *viewport) const
+{
+    qreal x(0.0), y( 0.0 );
+    viewport->screenCoordinates(m_point.coordinates(), x, y);
+    auto itemStyle = style();
+    if (itemStyle != 0 && !itemStyle->iconStyle().icon().isNull()) {
+        int halfIconWidth = itemStyle->iconStyle().icon().size().width() / 2;
+        int halfIconHeight = itemStyle->iconStyle().icon().size().height() / 2;
+
+        if ( x - halfIconWidth < curpos.x() &&
+             curpos.x() < x + halfIconWidth &&
+             y - halfIconHeight / 2 < curpos.y() &&
+             curpos.y() < y + halfIconHeight / 2 ) {
+            return true;
+        }
+    } else if ( curpos.x() == x && curpos.y() == y ) {
+        return true;
+    }
+
+    return false;
+}
+
 void GeoPhotoGraphicsItem::setPoint( const GeoDataPoint &point )
 {
     m_point = point;
