@@ -30,30 +30,30 @@ class PolylineAnnotation : public SceneGraphicsItem
 
 public:
     explicit PolylineAnnotation( GeoDataPlacemark *placemark );
-    ~PolylineAnnotation();
+    ~PolylineAnnotation() override;
 
     /**
      * @brief Paints the nodes on the screen and updates the regions which correspond
      * to each node using the given GeoPainter.
      */
-    virtual void paint( GeoPainter *painter, const ViewportParams *viewport, const QString &layer );
+    void paint( GeoPainter *painter, const ViewportParams *viewport, const QString &layer , int tileZoomLevel) override;
 
     /**
      * @brief Returns true if either the polyline's associated region or one of its nodes
      * contains the given QPoint. Note that the return value depends on the state.
      */
-    virtual bool containsPoint( const QPoint &eventPos ) const;
+    bool containsPoint( const QPoint &eventPos ) const override;
 
     /**
      * @brief It is used so far to remove the hover effect while being in the
      * AddingPolylineNodes state (@see SceneGraphicsItem::dealWithItemChange documentation).
      */
-    virtual void dealWithItemChange( const SceneGraphicsItem *other );
+    void dealWithItemChange( const SceneGraphicsItem *other ) override;
 
     /**
      * @brief Moves the whole polyline to the destination point.
      */
-    virtual void move( const GeoDataCoordinates &source, const GeoDataCoordinates &destination );
+    void move( const GeoDataCoordinates &source, const GeoDataCoordinates &destination ) override;
 
     /**
      * @brief Changes the busy state of the object according to @p enabled. It is mostly used
@@ -110,7 +110,7 @@ public:
     /**
      * @brief Provides information for downcasting a SceneGraphicsItem.
      */
-    virtual const char *graphicType() const;
+    const char *graphicType() const override;
 
 protected:
     /**
@@ -118,11 +118,11 @@ protected:
      * SceneGraphicsItem::sceneEvent() (@see Template Method pattern). Each of these
      * event handlers are structured according to the state.
      */
-    virtual bool mousePressEvent( QMouseEvent *event );
-    virtual bool mouseMoveEvent( QMouseEvent *event );
-    virtual bool mouseReleaseEvent( QMouseEvent *event );
+    bool mousePressEvent( QMouseEvent *event ) override;
+    bool mouseMoveEvent( QMouseEvent *event ) override;
+    bool mouseReleaseEvent( QMouseEvent *event ) override;
 
-    virtual void dealWithStateChange( SceneGraphicsItem::ActionState previousState );
+    void dealWithStateChange( SceneGraphicsItem::ActionState previousState ) override;
 
 private:
     /**
