@@ -10,6 +10,7 @@
 
 #include "GeoDataTrack.h"
 #include "GeoDataPlacemark.h"
+#include "GeoDataAccuracy.h"
 #include "PositionProviderPlugin.h"
 #include "PluginManager.h"
 #include "MarbleModel.h"
@@ -68,7 +69,7 @@ PlacemarkPositionProviderPluginTest::PlacemarkPositionProviderPluginTest() :
 PositionProviderPlugin *PlacemarkPositionProviderPluginTest::createUninitializedPlugin( const PluginManager *pluginManager )
 {
     foreach ( const PositionProviderPlugin *plugin, pluginManager->positionProviderPlugins() ) {
-        if ( plugin->nameId() == "Placemark" ) {
+        if (plugin->nameId() == QLatin1String("Placemark")) {
             PositionProviderPlugin *instance = plugin->newInstance();
             return instance;
         }
